"""CNP tool for handling CNP numbers."""

import argparse
import random
import sys
import time
from datetime import date

from .__init__ import Cnp, Gender, Region

VERSION = '1.1.0'
PROG_DESCRIPTION = "Cnpgen - CLI tool for handling CNP related tasks."
MIN_D_DATE_TS = time.mktime(time.strptime('1990-01-01', "%Y-%M-%d"))
MAX_D_DATE_TS = time.mktime(time.strptime('2099-12-31', "%Y-%M-%d"))

# set argparser
parser = argparse.ArgumentParser(description=PROG_DESCRIPTION, prog='cnpgen')
parser.add_argument(
    '--version',
    action='version',
    version=VERSION
)
parser.add_argument(
    '-c',
    action='store_true',
    default=False,
    help='Start new CNP wizard.',
)
parser.add_argument(
    '-g',
    metavar='amount',
    type=int,
    nargs=1,
    help='Generate random CNPs.'
)
parser.add_argument(
    '-i',
    metavar='CNP',
    type=str,
    nargs='+',
    help='Show CNP information.'
)

if len(sys.argv) == 1:
    parser.print_help()
    sys.exit(1)

args = parser.parse_args()

if args.c:
    print(
        'New CNP wizard.\n'
        'Please input required information. Defaults are in brackets.\n'
    )

    resident = True
    in_resident = input('Resident? [y/N]: ')
    if not len(in_resident) or in_resident == 'N' or in_resident == 'n':
        resident = False

    try:
        in_gender = input('Gender? [M]: ').upper()
        gender = Gender.M
        if len(in_gender):
            gender = Gender[in_gender]
    except KeyError:
        print('Value not allowed! Please use M or F. Default is M.')
        sys.exit(2)
    try:
        b_day = int(input('Birth day: '))
        b_month = int(input('Birth month: '))
        b_year = int(input('Birth year: '))
    except ValueError:
        print('Provided values for birth date are invalid. Must be numbers.')
        sys.exit(2)
    try:
        b_date = date(b_year, b_month, b_day)
    except ValueError:
        print('Invalid date.')
        sys.exit(2)
    try:
        region = Region[input('Region: ').title().replace('-', '_').replace(' ', '__')]
    except KeyError:
        print('Invalid region name.')
        sys.exit(2)
    try:
        serial = 1
        in_serial = input('Order number [1]: ')
        if len(in_serial):
            serial = int(in_serial)
    except ValueError:
        print('Serial must be number. Default is 1.')
        sys.exit(2)
    try:
        print(f'\nRESULT: {Cnp(gender, b_date, region, serial, resident)}')
    except Exception as err:
        print(err)
        sys.exit(2)

elif args.g:
    iterations = args.g[0]
    for _ in range(iterations):
        rand_gender = random.choice(list(Gender))
        rtime = MIN_D_DATE_TS + random.random() * (MAX_D_DATE_TS - MIN_D_DATE_TS)
        rand_b_date = date.fromtimestamp(rtime)
        rand_region = random.choice(list(Region))
        rand_serial = random.randint(1, 999)
        print(Cnp(rand_gender, rand_b_date, rand_region, rand_serial))

elif args.i:
    print("\n")
    header = 'CNP Information'
    print(header)
    for _cnp in args.i:
        print('=' * len(header))
        print(Cnp.info(_cnp), "\n")
else:
    parser.print_help()
    sys.exit(1)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='cnp-generator',
    version='1.1.0',
    url='https://github.com/myshy93/cnp-generator',
    author='Mihai Dinu',
    author_email='mihai.dinu93@gmail.com',
    license='MIT',
    keywords='cnp romania generator decoder',
    description='Romania CNP number generator and decoder.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=['cnpgen'],
    python_requires='>=3.6',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from datetime import datetime, date
from enum import Enum


class Gender(Enum):
    M = 1
    F = 2


class Region(Enum):
    Alba = 1
    Arad = 2
    Arges = 3
    Bacau = 4
    Bihor = 5
    Bistrita_Nasaud = 6
    Botosani = 7
    Brasov = 8
    Braila = 9
    Buzau = 10
    Caras_Severin = 11
    Cluj = 12
    Constanta = 13
    Covasna = 14
    Dambovita = 15
    Dolj = 16
    Galati = 17
    Gorj = 18
    Hargita = 19
    Hunedoara = 20
    Ialomita = 21
    Iasi = 22
    Ilfov = 23
    Maramures = 24
    Mehedinti = 25
    Mures = 26
    Neamt = 27
    Olt = 28
    Prahova = 29
    Satu_Mare = 30
    Salaj = 31
    Sibiu = 32
    Suceava = 33
    Teleorman = 34
    Timis = 35
    Tulcea = 36
    Vaslui = 37
    Valcea = 38
    Vrancea = 39
    Bucuresti = 40
    Bucuresti__Sector__1 = 41
    Bucuresti__Sector__2 = 42
    Bucuresti__Sector__3 = 43
    Bucuresti__Sector__4 = 44
    Bucuresti__Sector__5 = 45
    Bucuresti__Sector__6 = 46
    Calarasi = 51
    Giurgiu = 52

    def __str__(self):
        normalize = self.name.replace('__', ' ').replace('_', '-')
        return normalize

class Cnp:
    LENGTH = 13
    PARTIAL_LENGTH = LENGTH - 1
    REF_CONSTANT = [2, 7, 9, 1, 4, 6, 3, 5, 8, 2, 7, 9]

    def __init__(self, gender, birth_date, region, serial=1, resident=False):
        """Cnp is a object that holds data needed to construct a CNP number.

        :param gender: Person's gender.
        :type gender: Gender
        :param birth_date: Person's birth date
        :type birth_date: date
        :param region: Region of birth
        :type region: Region
        :param serial: Order number for persons born in same day an same region.
        Default is 1.
        :type serial: int
        :param resident: Whether person is a resident or not. Default is false.
        :type resident: bool
        :raises TypeError: If any of parameters are not of required type or in range.
        """
        # sanity checks
        if not isinstance(gender, Gender):
            raise TypeError('Gender in not a gender object.')

        if not isinstance(birth_date, date):
            raise TypeError('Birth date is not a datetime object.')

        if not isinstance(region, Region):
            raise TypeError('Region is not a Region object.')

        if not isinstance(serial, int) or serial > 1000 or serial < 1:
            raise TypeError('Invalid serial. Must be a int in range 1-999.')

        if not isinstance(resident, bool):
            raise TypeError('Residential state should be bool.')

        self.gender = gender
        self.birth_date = birth_date
        self.region = region
        self.serial = serial
        self.resident = resident

    def __str__(self):
        return self.full

    def __repr__(self):
        return f"< Cnp {self.full} >"

    def get_gender_code(self):
        """Calculate gender code based on gender, year and whether is resident or not.

        :raises ValueError: Invalid year was provided to constructor.
        :return: Gender code
        :rtype: int
        :raises ValueError: If year in not in range 1900 - 2099.
        """
        if self.resident:
            gen_code = 7
        elif 1900 <= self.birth_date.year <= 1999:
            gen_code = 1
        elif 2000 <= self.birth_date.year <= 2099:
            gen_code = 5
        else:
            raise ValueError('Year is out of range. Allowed range 1900-2099')

        if self.gender == Gender.F:
            gen_code += 1

        return gen_code

    @property
    def partial(self):
        """Put together first 12 digits of CNP based on CNP standard.

        :return: First 12 digits of CNP.
        :rtype: str
        """
        return "".join([
            str(self.get_gender_code()),
            self.birth_date.strftime("%y"),
            self.birth_date.strftime("%m"),
            self.birth_date.strftime("%d"),
            f"{self.region.value:0>2}"
            f"{self.serial:0>3}",
        ])

    @property
    def full(self):
        """Full CNP as string.

        :return: Full CNP as string.
        :rtype: str
        """
        return f"{self.partial}{self.compute_c(self.partial)}"

    @staticmethod
    def compute_c(partial):
        """Compute check digit for a partial CNP (12 digits).

        :param partial: Partial CNP to check.
        :type partial: str
        :return: Calculated control digit.
        :rtype: int
        :raises TypeError: If partial cnp provided is not a string or it length
        is not equal to Cnp.PARTIAL_LENGTH.
        :raises Value error: If provided partial contains other chars then digits.
        """
        if not isinstance(partial, str) or len(partial) != Cnp.PARTIAL_LENGTH:
            raise TypeError("Invalid partial CNP, expected a string with length 12.")

        digits = [int(x) for x in partial if x.isdigit()]

        if len(digits) != len(partial):
            raise ValueError('Partial must contain only digits!')

        step1 = [digits[i] * Cnp.REF_CONSTANT[i] for i in range(len(digits))]
        step2 = sum(step1) % 11

        if step2 < 10:
            return step2
        else:
            return 1

    @staticmethod
    def is_valid(_cnp):
        """Check CNP validity.

        :param _cnp: CNP to check.
        :type _cnp: str
        :return: True if CNP is valid of False if not.
        :rtype: bool
        """
        if isinstance(_cnp, str) and len(_cnp) == Cnp.LENGTH:
            partial = _cnp[:-1]
            if int(_cnp[-1]) == Cnp.compute_c(partial):
                return True
        return False

    @staticmethod
    def info(_cnp):
        """Builds a formatted string after decoding information contained in CNP.

        :param _cnp: CNP to decode.
        :return: Decoded info.
        :rtype: str
        """
        if Cnp.is_valid(_cnp):
            if int(_cnp[0]) in [1, 2]:
                gender = Gender(int(_cnp[0]))
                year_prefix = "19"
            else:
                gender = Gender(int(_cnp[0]) - 4)
                year_prefix = "20"
            b_date = datetime.strptime(f"{year_prefix}{_cnp[1:7]}", "%Y%m%d")
            region = Region(int(_cnp[7:9]))
            return (f"CNP: {_cnp}\n"
                    f"Gender: {gender.name}\n"
                    f"Birth date: {b_date.date().strftime('%d.%m.%Y')}\n"
                    f"Region: {region}\n"
                    )
        else:
            raise ValueError('Invalid CNP.')
